#define busy_width 16
#define busy_height 16
#define busy_x_hot -1
#define busy_y_hot -1
static unsigned char busy_bits[] = {
   0xee, 0xee, 0xff, 0xff, 0xbb, 0xbb, 0xff, 0xff, 0xee, 0xee, 0xff, 0xff,
   0xbb, 0xbb, 0xff, 0xff, 0xee, 0xee, 0xff, 0xff, 0xbb, 0xbb, 0xff, 0xff,
   0xee, 0xee, 0xff, 0xff, 0xbb, 0xbb, 0xff, 0xff};
